<?php

namespace App\Service;

use App\Dto\Player;
use Symfony\Component\Process\Process;

class Factorio
{
    public function __construct(
        private readonly string $initPath = '/home/ardent/games/factorio-init/factorio',
    ) {}

    private function runBase(array|string $args): array
    {
        if (is_string($args)) {
            $args = [$args];
        }

        $process = new Process([$this->initPath, ...$args]);
        $process->run();

        return explode(PHP_EOL, $process->getOutput());
    }

    private function runCmd(array|string $args): array
    {
        if (is_string($args)) {
            $args = [$args];
        }

        return $this->runBase(['cmd', ...$args]);
    }

    /**
     * @return Player[]
     */
    public function getOnlinePlayers(): array
    {
        $output = $this->runBase('players');
        array_shift($output); // Remove header
        array_pop($output); // Remove empty line

        $players = [];
        foreach ($output as $line) {
            $players[] = new Player(explode(' ', trim($line))[0]);
        }

        return $players;
    }

    public function kickPlayer(Player $player, string $reason = ''): void
    {
        $this->runCmd(['/kick', $player->getName(), $reason]);
    }

    public function whisper(Player $player, string $message): void
    {
        $this->runCmd(['/whisper', $player->getName(), $message]);
    }

    public function say(string $message): void
    {
        $this->runCmd($message);
    }
}