<?php

namespace App\Service\Rules;

use App\Dto\Player;
use App\Service\SimpleDataService;

class PlayTimeRule extends AbstractRule
{
    public function __construct(
        private readonly SimpleDataService $storeData,
        private readonly int               $defaultPlayTime = 60,
        private readonly int               $timeBetweenChecks = 1,
    ) {}

    private function timePerPlayer(Player $player)
    {
        $players = [
//            "Ardentsword" => "2:00",
        ];

        return $players[$player->getName()] ?? $this->defaultPlayTime;
    }

    public function check(Player $player): RuleResult
    {
        $today = new \DateTimeImmutable();
        $playerData = $this->getPlayerData($player);

        // If the player has no data or the data is from a different day, we reset the time
        if (!$playerData || $playerData['day'] !== $today->format('Y-m-d')) {
            $this->storePlayerData($player, $today, $this->timePerPlayer($player));
            return new RuleResult(RuleResultEnum::WARNING, sprintf('Extra time activated, set to %d minutes', $this->timePerPlayer($player)));
        }

        // If the player has time left, we decrease it and return denied if nothing left
        $timeLeft = $playerData['timeLeft'] - $this->timeBetweenChecks;
        $this->storePlayerData($player, $today, $timeLeft);

        if ($timeLeft <= 0) {
            return new RuleResult(RuleResultEnum::DENIED);
        } elseif ($timeLeft <= 5) {
            return new RuleResult(RuleResultEnum::WARNING, sprintf('%d minute(s) left', $timeLeft));
        }elseif($timeLeft === 30) {
            return new RuleResult(RuleResultEnum::WARNING, sprintf('30 minutes left'));
        }

        return new RuleResult(RuleResultEnum::ALLOWED);
    }

    public function getName(): string
    {
        return sprintf('PlayTimeRule, limited play time');
    }

    private function storePlayerData(Player $player, \DateTimeImmutable $day, int $timeLeft): void
    {
        $this->storeData->set(
            sprintf('playTime-%s', $player->getName()),
            [
                'player' => $player->getName(),
                'timeLeft' => $timeLeft,
                'day' => $day->format('Y-m-d'),
            ]);
    }

    private function getPlayerData(Player $player): ?array
    {
        return $this->storeData->get(
            sprintf('playTime-%s', $player->getName())
        );
    }
}