<?php

namespace App\Service\Rules;

use App\Dto\Player;

class TimeRule extends AbstractRule
{
    public function __construct(
        private readonly string $start = '19:00',
        private readonly string $end = '23:00',
    ) {}

    public function check(Player $player): RuleResult
    {
        $now = new \DateTimeImmutable();
        $start = new \DateTimeImmutable($this->start);
        $end = new \DateTimeImmutable($this->end);

        if ($now >= $start && $now <= $end) {
            $fiveMinutesBeforeEnd = $end->sub(new \DateInterval('PT5M'));
            if ($now >= $fiveMinutesBeforeEnd) {
                return new RuleResult(RuleResultEnum::WARNING, 'Only 5 minutes remaining');
            }
            return new RuleResult(RuleResultEnum::ALLOWED);
        }

        return new RuleResult(RuleResultEnum::DENIED, 'After designed hours');
    }

    public function getName(): string
    {
        return sprintf('TimeRule, time is between %s and %s', $this->start, $this->end);
    }
}